#
# Licensed Materials - Property of IBM
#
# Rational ClearQuest
#
# (C) Copyright IBM Corp. 2005,2006  All Rights Reserved
#

package Long2clob::Globals;

use strict;
no warnings qw/once/;
use Exporter;
our @ISA=qw/Exporter/;
our @EXPORT_OK = qw();

use vars qw($VERSION);
$VERSION = '1.03';

######################################################################
# Misc constants
push @EXPORT_OK, qw($LONG2CLOB);
our $LONG2CLOB = "LONG2CLOB";

######################################################################
# Status messages
push @EXPORT_OK, qw($STM_NOT_LOCKED $STM_NOT_INITIALIZED
                    $STM_PLAN $STM_EXECUTE $STM_ANALYZE);
our $STM_STATUS = "STATUS";
our $STM_NOT_LOCKED = "STATUS: Database not locked.\n";
our $STM_NOT_INITIALIZED = "STATUS: Conversion not initialized.\n";
our $STM_ANALYZE = "Analyzing";
our $STM_EXECUTE = "Executing";
our $STM_PLAN = "Planning";

######################################################################
# Error messages

push @EXPORT_OK, qw($ERM_FINALIZE
                    $ERM_INTERNAL $ERM_LOCK_FAILED $ERM_INVALID_SUBCOMMAND
                    $ERM_MISSING_SUBCOMMAND $ERM_MISSING_OPTION
                    $ERM_NOT_INITIALIZED $ERM_OPEN_READ $ERM_OPERATIONS
                    $ERM_ORACLE $ERM_PDSQL $ERM_PDSQL_SELECT
                    $ERM_UNLOCK_FAILED $ERM_USAGE);
our $ERM_FINALIZE = "*** Finalize Error";
our $ERM_INTERNAL = "*** Internal Error";
our $ERM_INVALID_SUBCOMMAND = "*** Usage Error\nInvalid subcommand";
our $ERM_LOCK_FAILED = "*** Lock Error\nThe database could not be locked";
our $ERM_MISSING_SUBCOMMAND = "*** Usage Error\nMissing subcommand";
our $ERM_MISSING_OPTION = "*** Usage Error\nMissing database parameter";
our $ERM_NOT_INITIALIZED = "*** Conversion Errror\nThe conversion process must be initialized";
our $ERM_OPERATIONS = "*** Conversion Error";
our $ERM_ORACLE = "*** Oracle Error";
our $ERM_PDSQL = "*** PDSQL Error";
our $ERM_PDSQL_SELECT = "*** PDSQL Select Error";
our $ERM_USAGE = "*** Usage Error";

######################################################################
# SR and UserDB HASH tables containing ARRAYs describing the LOB tables that
# need to be created when converting from LONG to CLOB.
#
# The two hashes have 5 ARRAYs with one array entry per LOB table to create.
# The keys are:
#
#   'table_name'    = names of the LONG tables to replace with LOB tables
#   'column_create' = LOB table column definitions
#   'select_clause' = how to insert into the LOB table from the LONG table
#   'index_name'    = names of the indexes on each table
#   'index_create'  = how to create the indices on the LOB tables
#
######################################################################
push @EXPORT_OK, qw(%Fixed_SR_Info %Fixed_UserDB_Info);
our %Fixed_SR_Info =
  (
   table_name=>
   [
    "MASTER_DBS", "MASTER_GLOBAL", "M_FORMDEF",
    "M_PROPERTY", "M_SCRIPTDEF", "MASTER_OPLOG",
    "MASTER_PROPERTIES", "SCHEMAREV_BINARIES",
    "SCHEMAREV_TEXTS"
   ],
   column_create=>
   [
    "master_dbid number(10), version number(10), db_num number(10), name varchar2(5), description varchar2(80), vendor number(10), server varchar2(30), database_name varchar2(255), table_prefix varchar2(100), metaschema_version number(10), feature_level number(10), schema_id number(10), schema_rev number(10), schemarev_version number(10), initialized number(10), is_master number(10), is_deleted number(10), db_ownership number(10), is_visible number(10), f1 varchar2(255), f2 varchar2(255), connect_options clob",
    "site_id number(10), site_name varchar2(5), set_name varchar2(255), metaschema_version number(10), feature_level number(10), next_master_id number(10), next_db_num number(10), next_schema_num number(10), next_package_id number(10), next_package_ownership_id number(10), next_package_script_id number(10), last_request_id number(10), last_aux_id number(10), working_master varchar2(36), site_uuid varchar2(36), check_timeout_interval number(10), timeout_interval number(10), db_locked number(10), db_locked_info varchar2(255), next_oplog_serial_no number(10), next_uuid_index number(10), roll_user_ids_script clob",
    "id number(10), version number(10), schema_id number(10), schema_rev number(10), package_ownership number(10), name varchar2(255), entitydef_id number(10), is_default number(10), form_blob blob",
    "id number(10), version number(10), schema_id number(10), schema_rev number(10), package_ownership number(10), name varchar2(255), data clob",
    "id number(10), version number(10), schema_id number(10), schema_rev number(10), package_ownership number(10), hookdef_id number(10), language number(10), program clob",
    "serial_no number(10), replay_time date, replica_time date, replica_id number(10), epoch_no number(10), schema_id number(10), schema_rev number(10), schemarev_version number(10), type number(10), packlet_len number(10), packlet blob",
    "master_dbid number(10), version number(10), ratl_mastership number(10), name varchar2(255), data clob",
    "schema_id number(10), schema_rev number(10), id number(10), data blob",
    "schema_id number(10), schema_rev number(10), id number(10), data clob"
   ],
   select_clause=>
   [
    "master_dbid, version, db_num, name, description, vendor, server, database_name, table_prefix, metaschema_version, feature_level, schema_id, schema_rev, schemarev_version, initialized, is_master, is_deleted, db_ownership, is_visible, f1, f2, TO_LOB(connect_options)",
    "site_id, site_name, set_name, metaschema_version, feature_level, next_master_id, next_db_num, next_schema_num, next_package_id, next_package_ownership_id, next_package_script_id, last_request_id, last_aux_id, working_master, site_uuid, check_timeout_interval, timeout_interval, db_locked, db_locked_info, next_oplog_serial_no, next_uuid_index, TO_LOB(roll_user_ids_script)",
    "id, version, schema_id, schema_rev, package_ownership, name, entitydef_id, is_default, TO_LOB(form_blob)",
    "id, version, schema_id, schema_rev, package_ownership, name, TO_LOB(data)",
    "id, version, schema_id, schema_rev, package_ownership, hookdef_id, language, TO_LOB(program)",
    "serial_no, replay_time, replica_time, replica_id, epoch_no, schema_id, schema_rev, schemarev_version, type, packlet_len, TO_LOB(packlet)",
    "master_dbid, version, ratl_mastership, name, TO_LOB(data)",
    "schema_id, schema_rev, id, TO_LOB(data)",
    "schema_id, schema_rev, id, TO_LOB(data)"
   ],
   index_name=>
   [
    [
     "MASTER_DBS_IDS",
     "MASTER_DBS_NAME"
    ],
    [],
    "M_FORMDEF_ID",
    "M_PROPERTY_ID",
    "M_SCRIPTDEF_ID",
    "MASTER_OPLOG_INDEX",
    [
     "MASTER_PROPERTIES_ID",
     "MASTER_PROPERTIES_NAME"
    ],
    "SCHEMAREV_BINARIES_IX",
    "SCHEMAREV_TEXTS_IX"
   ],
   index_create=>
   [
    [
     "create unique index master_dbs_ids ON master_dbs (master_dbid);",
     "create unique index master_dbs_name ON master_dbs (name);"
    ],
    [],
    "create unique index m_formdef_id ON m_formdef (schema_id, schema_rev, id);",
    "create unique index m_property_id ON m_property (schema_id, schema_rev, id);",
    "create unique index m_scriptdef_id ON m_scriptdef (schema_id, schema_rev, id);",
    "create unique index master_oplog_index ON master_oplog (serial_no);",
    [
     "create unique index master_properties_id ON master_properties (master_dbid);",
      "create unique index master_properties_name ON master_properties (name);"
    ],
    "create unique index schemarev_binaries_ix ON schemarev_binaries (schema_id, schema_rev, id);",
    "create unique index schemarev_texts_ix ON schemarev_texts (schema_id, schema_rev, id);"
   ]
  );

our %Fixed_UserDB_Info =
  (
   table_name=>
   [
    "ATTACHMENTS_BLOB", "FORMDEF", "PROPERTY", "SCRIPTDEF", "RATL_OPLOG", "USER_BLOB", "MULTILINE_TEXT"
   ],
   column_create=>
   [
    "attachments_dbid number(10), entity_dbid number(10), entity_fielddef_id number(10), data blob",
    "id number(10), version number(10), schema_id number(10), schema_rev number(10), package_ownership number(10), name varchar2(255), entitydef_id number(10), is_default number(10), form_blob blob",
    "id number(10), version number(10), schema_id number(10), schema_rev number(10), package_ownership number(10), name varchar2(255), data clob",
    "id number(10), version number(10), schema_id number(10), schema_rev number(10), package_ownership number(10), hookdef_id number(10), language number(10), program clob",
    "serial_no number(10), replay_time date, replica_time date, replica_id number(10), epoch_no number(10), schema_id number(10), schema_rev number(10), schemarev_version number(10), type number(10), packlet_len number(10), packlet blob",
    "dbid number(10), data blob",
    "entity_dbid number(10), entity_fielddef_id number(10), data clob"
   ],
   select_clause=>
   [
    "attachments_dbid, entity_dbid, entity_fielddef_id, TO_LOB(data)",
    "id, version, schema_id, schema_rev, package_ownership, name, entitydef_id, is_default, TO_LOB(form_blob)",
    "id, version, schema_id, schema_rev, package_ownership, name, TO_LOB(data)",
    "id, version, schema_id, schema_rev, package_ownership, hookdef_id, language, TO_LOB(program)",
    "serial_no, replay_time, replica_time, replica_id, epoch_no, schema_id, schema_rev, schemarev_version, type, packlet_len, TO_LOB(packlet)",
    "dbid, TO_LOB(data)",
    "entity_dbid, entity_fielddef_id, TO_LOB(data)"
   ],
   index_name=>
   [
    "ATTACHMENTS_BLOB",
    [
     "FORMDEF_ID",
     "FORMDEF_ENTITY"
    ],
    [],
    [],
    "RATL_OPLOG_IX",
    "USER_BLOB_DBID",
    "MLTEXT_ENTITY"
   ],
   index_create=>
   [
    "create unique index attachments_blob ON attachments_blob (attachments_dbid);",
    [
     "create unique index formdef_id ON formdef (id);",
     "create index formdef_entity ON formdef (entitydef_id);"
    ],
    [],
    [],
    "create unique index ratl_oplog_ix ON ratl_oplog (serial_no);",
    "create unique index user_blob_dbid ON user_blob (dbid);",
    "create unique index mltext_entity on multiline_text (entity_dbid, entity_fielddef_id);"
   ]
  );

1;

# Local Variables: #
# mode:cperl #
# comment-column:40 #
# cperl-comment-column:40 #
# cperl-continued-brace-offset:-2 #
# cperl-indent-level:4 #
# indent-tabs-mode:nil #
# fill-column: 78 #
# End: #
