@rem = '--*-Perl-*--';
@rem = '
@echo off
if exist %0\NUL goto next1
if not exist %0 goto next1
cqperl %0 %*
goto :EOF
:next1
if not exist %0.bat goto next2
cqperl %0.bat %*
goto :EOF
:next2
cqperl -S %0 %*
goto :EOF
@rem ';

#
# Licensed Materials - Property of IBM
#
# IBM Rational ClearQuest
#
# (C) Copyright IBM Corp. 2006  All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

package main;

=head1 NAME

 long2clob - Converts a Rational ClearQuest database from LONG to CLOB

=head1 SYNOPSIS

 long2clob {-man|-version}
 long2clob <db-options> lock
 long2clob <db-options> initialize
 long2clob <db-options> status [-verbose]
 long2clob <db-options> convert [-fast] [-verbose]
 long2clob <db-options> finalize
 long2clob <db-options> finalize <master-db-options>
 long2clob <db-options> unlock
 long2clob <db-options> cleanup

=head1 DESCRIPTION

The long2clob command provides a set of subcommands to manage the conversion
of Rational ClearQuest databases from LOB_TYPE=LONG (which uses the deprecated
LONG and LONG RAW datatypes) to LOB_TYPE=CLOB (which uses CLOB and BLOB
datatypes).

This utility has been tested only on systems with Rational ClearQuest
v2003.06.12 or later clients installed.  It does not work with version 7.0
client.

To ensure that the databases are backed up successfully, you must restart each
database server to disconnect existing clients after you issue the lock
subcommand and before you make the backup.  After the conversion process has
started, the database cannot be used until the conversion is complete or the
database has been restored from backup.

Some code pages are not supported by Rational ClearQuest v2003.06 clients with
CLOB databases.  To avoid any risk that a client can corrupt characters in a
database that uses an unsupported code page, you must completely upgrade the
environment while clients are disconnected.  To do this, lock all databases,
upgrade all clients on all hosts to version 7.0, and convert all databases to
CLOB.  After these steps are completed, re-enable client access by using the
unlock subcommand on the converted databases.  For more information, see the
"IBM Rational ClearQuest and ClearQuest MultiSite Installation and Upgrade
Guide."

Queries on the MULTILINE_TEXT table that were modified in the SQL editor pane
fail after conversion.  Re-create them in the Query editor pane.

=head1 DATABASE OPTIONS

The command uses a different set of database options, depending on which
version of Rational ClearQuest is installed.

<db-options> for Rational ClearQuest v2003.06.12:

    -v {ora7|ora8} -u <userid> -p <password> -s <alias> -co <connect-options>

<db-options> for Rational ClearQuest v2003.06.13 or later:

    -u <userid> -p <password> -db <sid> -s <host>

=over 4

=item -v {ora7|ora8}

On hosts with Rational ClearQuest v2003.06.12, this is a required parameter
specifying the database server version.

=item -u <userid>

The database instance user ID.

=item -p <password>

The database instance user password.

=item -s <host>

=item -s <alias>

On hosts with Rational ClearQuest v2003.06.12, this specifies a TNS-NAMES
alias.  On hosts with Rational ClearQuest v2003.06.13 or later, this specifies the
database server host where database server instance is running.

=item -db <sid>

The database instance system identifier.

=item -co <connect-options>

The database connection parameters.  To determine the correct value for a
master database, start the Rational ClearQuest Maintenance Tool and select the
database.  Use the value in the Connect Options box.  To determine the correct
value for a user database, start the Rational ClearQuest Designer and click
Database > View Database Properties.  Select the user database and click
Properties.  Use the value in the Connect Options box.  Note: The connect
options must be gathered before locking the master database.

=back

=head1 COMMAND OPTIONS

=over 4

=item -man

Prints the complete manual page and exits.

=item -verbose

Provides more verbose output from subcommands.

=item -version

Displays the version information for this utility and exits.

=back

=head1 SUBCOMMANDS

=over 4

=item cleanup

The cleanup subcommand performs any remaining conversion operations deferred
by convert -fast and drops all temporary data created for tracking the
conversion process.

=item convert

The convert subcommand executes conversion operations from the tracking table
created by the initialize subcommand.  The -verbose option causes time stamps
and SQL statements to be printed with each operation.  The -fast option causes
the conversion of attachments and the purging of temporary data to be deferred
to the cleanup subcommand.  Users who attempt to open a deferred attachment
receive a warning that the attachment "may not be committed" and cannot open
it.  If the process is interrupted, rerun the convert subcommand to execute
remaining operations.

=item finalize

The finalize subcommand changes the LOB_TYPE entry for the database from LONG
to CLOB in the master database.  If a user database is being converted, this
subcommand requires the options for the master database as a second set of
parameters.

To finalize a master database:

  long2clob <master-db-options> finalize

To finalize a user database:

  long2clob <user-db-options> finalize <master-db-options>

=item initialize

The initialize subcommand analyzes a locked LONG database to convert to
CLOB.  It creates the database table and sequence object for tracking the
conversion process.

=item lock

The lock subcommand locks a Rational ClearQuest LONG database for conversion
to CLOB.  After you issue this subcommand, you must restart the database
server to disconnect existing connections and back up it up.

=item status

The status subcommand prints the planned conversion operations and the
conversion status; it also prints the next subcommand to use in the conversion
process.  If the -verbose option is included, it also prints the SQL statement
associated with each operation.

=item unlock

The unlock subcommand unlocks a LONG database after it is restored from backup
or a CLOB database after its connect_options have been updated by the finalize
subcommand.

=back

=head1 EXAMPLE

To convert all databases at a site at once, preventing users from accessing
any databases until all databases have been converted and all clients have
been upgraded, follow these steps.  This procedure avoids the risk that a
client can corrupt characters in a database that uses a code page that is
unsupported with the v2003.06.12 and later clients.

This can be a time-consuming procedure.  To save time:

  * Perform steps 2, 3, and 4 in parallel.
  * Use the -fast option with the convert subcommand.

=over 4

=item 1.

Lock and back up all master and user databases.  Run these commands for every
master and user database; then restart and back up all servers.

 long2clob <db-options> lock

=item 2.

Convert all master databases and leave them locked.  Run these commands for
every master database:

 long2clob <master-db-options> initialize
 long2clob <master-db-options> convert
 long2clob <master-db-options> finalize

=item 3.

Convert every user database, leaving them unlocked but inaccessible because
the master databases are still locked.  Run these commands for every user
database:

 long2clob <user-db-options> lock
 long2clob <user-db-options> initialize
 long2clob <user-db-options> convert
 long2clob <user-db-options> finalize <master-db-options>
 long2clob <user-db-options> unlock

=item 4.

Upgrade all Rational ClearQuest clients on all hosts to v7.0.

=item 5.

Unlock the master databases and clean up temporary data.  Run these commands
for every master database.

 long2clob <master-db-options> unlock
 long2clob <master-db-options> cleanup

=item 6.

Perform any conversions deferred by convert -fast and clean up temporary data
in the user databases.

 long2clob <user-db-options> cleanup

=cut

use strict;
use Data::Dumper;
use File::Basename;
use File::Path;
use Getopt::Long qw(:config require_order);
use IO::File;
use Pod::Usage qw();

use FindBin;
use lib "$FindBin::RealBin";
use Long2clob::Globals qw(/\$ERM_/);
use Long2clob::Subcommand qw/%COMMANDS/;

use vars qw($VERSION);
$VERSION = '1.20';

sub VersionMessage;                     # print the version message and exit
sub pod2usage(%);                       # work around for missing perldoc
select(STDERR);
$|=1;
select(STDOUT);
$|=1;

######################################################################
# Main program
######################################################################

# Define variables to hold command line options
#
use Long2clob::Diag qw/%opt_diag/;          
our %db;
our $opt_subcommand;            # name of the subcommand being executed
our $opt_verbose;               # -verbose: increase output
our $opt_fatal;                 # -fatal  : shuts off error handler (dev only!)
our $opt_noop;                  # -noop   : dont really execute ops (dev only!)

# Look for -verbose, -fatal, and -noop flags anywhere in the command line.
#
@ARGV = grep { ! ($_ eq '-verbose' and $opt_verbose = 1) } @ARGV;
@ARGV = grep { ! ($_ eq '-fatal' and $opt_fatal = 1) } @ARGV;
@ARGV = grep { ! ($_ eq '-noop' and $opt_noop = 1) } @ARGV;

# Read command line options stopping at the subcommand name.
#
GetOptions('help|h'   => sub { pod2usage(-verbose=>0, -exitval=>0); },
           'man|m'    => sub { pod2usage(-verbose=>2, -exitval=>0); },
           'version'  => sub { VersionMessage(); exit(0); },
           'fatal'    => \$opt_fatal,   # -fatal (errors are fatal)
           'diag|d=s%'=> \%opt_diag,    # -diag subr=[-+0-9]
           'u=s'      => \$db{'u'},     # -u <user>
           'v=s'      => \$db{'v'},     # -v <oraver>
           'p=s'      => \$db{'p'},     # -p <pass>
           's=s'      => \$db{'s'},     # -s <sr3-host> or <sr2-alias>
           'db=s'     => \$db{'db'},    # -db <sid>
           'co=s'     => \$db{'co'})    # -co <connect_options>
  or pod2usage(-verbose=>0, -exitval=>2);

# Create a pdsql database access object using the command line options
#
my $pdsql;
eval {
    $pdsql = new Long2clob::PDSQL(%db);
};
pod2usage(-msg=>"$@\nFor the complete documentation, type 'long2clob -man'.")
 if $@;

# Read the subcommand name from the command line
#
$opt_subcommand = shift @ARGV
  or pod2usage(-msg=>$ERM_MISSING_SUBCOMMAND);

# Look up the corresponding subroutine ref in %long2clob::COMMAND
#
my $subr = $COMMANDS{$opt_subcommand}
  or pod2usage(-msg=>"$ERM_INVALID_SUBCOMMAND: '$opt_subcommand'");

# Create a subcommand dispatcher - the code for $subr lives here.
#
my $subcmd = new Long2clob::Subcommand(pdsql=>$pdsql);

# Execute the subcommand with an alternate syntax for $subcmd->$subr(@ARGV)
#
my $status = &$subr($subcmd, @ARGV);

# Subcommands should never return.
#
die "warning: subcommand '$opt_subcommand' returned status $status\n";

######################################################################
# End of main
######################################################################

sub VersionMessage {
    ########################################
    # Print version info for this script, the Long2clob modules, and cqperl.exe
    ########################################
    print STDERR "$0 version $VERSION\n";
    my @files = grep { /\bLong2clob\b/i } keys %INC;
    for my $file (@files) {
        my $module = $file;
        $module =~ s/[\/\\]/::/g;
        $module =~ s/\.pm$//;
        my $version = '$'.$module.'::VERSION';
        $version = eval $version;
        my $path = $INC{$file};
        $path =~ s/\\/\//g;
        print STDERR "  $path version $version\n";
    }
    my ($perlExe, $perlVer) = ( $^X, $] );
    print STDERR "  $^X version ", $]>=5.006 ? sprintf("%vd", $^V) : $], "\n";
}

sub pod2usage(%) {
    ########################################
    # Call Pod::Usage::pod2usage with (-input=>HANDLE), otherwise it
    # will fail when generating a man page because the script
    # "$Config{bin}/perldoc" is not included with ratlperl.
    ########################################
    my (%p) = (@_);
    if (! exists $p{-input}) {
        my $FH = new IO::File("$FindBin::RealBin\\$FindBin::RealScript", "r")
          || die "$ERM_INTERNAL: reading $FindBin::RealBin\\$FindBin::RealScript: $!\n ";
        Pod::Usage::pod2usage(-input=>$FH, @_);
    }
    else {
        Pod::Usage::pod2usage(@_);
    }
}

1;

# Local Variables: #
# mode:cperl #
# comment-column:40 #
# cperl-comment-column:40 #
# cperl-continued-brace-offset:-2 #
# cperl-indent-level:4 #
# indent-tabs-mode:nil #
# fill-column: 78 #
# End: #
